<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking settings class
 * Administrative contents
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Admin
{
    /**
     * The single instance of the class.
     */
    public static $instance;

    /**
     * Create & return an instance of the class.
     *
     * @return object OHBE_Admin instance.
     */
    public static function getInstance() {
        if (null == self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Add hooks to initialize the class.
     *
     * @return void
     */
    public function __construct() {
        // Add Redforts admin menu
        add_action('admin_menu', array($this, 'registerAdminMenu'));
        // Register options
        add_action('admin_init', array($this, 'registerSettings'));
        // Save options
        add_action('admin_init', array($this, 'saveSettings'));
        // CSS & JS for Color Picker
        add_action('admin_enqueue_scripts', array($this, 'loadAssets'), 21);
    }

    /**
     * Add the settings page to the wp admin menu.
     *
     * @return void
     */
    public function registerAdminMenu() {
        $p = add_menu_page(
            'Oscar',
            'Oscar Hotel',
            'manage_options',
            'ohbe',
            array($this, 'adminPage'),
            OHBE_URL.'assets/images/redforts-icon.png',
            90
        );
        add_action('load-' . $p, array(__CLASS__, 'addHelpTabs'));
    }

    /**
     * Display the options page for the plugin.
     *
     * @return void
     */
    public function adminPage() {
        if (!current_user_can('manage_options')) {
            return;
        }
        printf(
            '<div class=wrap><h1>%s</h1>',
            __('Oscar Hotel Settings', 'ohbe')
        );
        if (isset($_GET['settings-updated'])) {
            add_settings_error(
                'wporg_messages',
                'wporg_message',
                __('Settings saved.', 'ohbe'),
                'updated'
            );
        }
        settings_errors('wporg_messages');
        print '<form method=POST action=options.php>';
        print '<input type=hidden name=action>';
        settings_fields('oscar-hotel-settings');
        do_settings_sections('oscar-hotel-settings');
        submit_button();
        wp_nonce_field('update_ohbe_settings', 'update_ohbe_settings');
        print '</form>';
    }

    /**
     * Register plugin settings.
     *
     * @return void
     */
    public function registerSettings() {
        // Register settings array name
        register_setting(
            'oscar-hotel-settings',
            'ohbe',
            array($this, 'sanitizeInput')
        );
        // Settings section: Main Settings
        add_settings_section(
            'main-settings',
            '',
            array($this, 'mainSettingsCallback'),
            'oscar-hotel-settings'
        );
        // Connetion_code
        add_settings_field(
            'connection_code',
            __('Connection code', 'ohbe'),
            array($this, 'clientCodeCallback'),
            'oscar-hotel-settings',
            'main-settings'
        );
        // Settings section: Booking
        add_settings_section(
            'booking-settings',
            __('Reservation', 'ohbe'),
            null,
            'oscar-hotel-settings'
        );
        // Test_mode
        add_settings_field(
            'test_mode',
            __('Test mode', 'ohbe'),
            array($this, 'testModeCallback'),
            'oscar-hotel-settings',
            'booking-settings'
        );
        // Notes
        add_settings_field(
            'notes',
            __('Activate notes field', 'ohbe'),
            array($this, 'notesCallback'),
            'oscar-hotel-settings',
            'booking-settings'
        );
        // Conditions
        add_settings_field(
            'conditions',
            __('Show conditions opt in text', 'ohbe'),
            array($this, 'conditionsCallback'),
            'oscar-hotel-settings',
            'booking-settings'
        );
        // Settings section: Availability
        add_settings_section(
            'availability-settings',
            __('Availability', 'ohbe'),
            null,
            'oscar-hotel-settings'
        );
        // Only left number
        add_settings_field(
            'only-left',
            __('Only left: quantity', 'ohbe'),
            array($this, 'onlyLeftCallback'),
            'oscar-hotel-settings',
            'availability-settings'
        );
        // Settings section: Colors
        add_settings_section(
            'color-settings',
            __('Colors', 'ohbe'),
            null,
            'oscar-hotel-settings'
        );
        // Btn font color
        add_settings_field(
            'btn-font-color',
            __('Buttons font', 'ohbe'),
            array($this, 'btnFontColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Btn background color
        add_settings_field(
            'btn-bg-color',
            __('Buttons background', 'ohbe'),
            array($this, 'btnBgColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Promo tag font color
        add_settings_field(
            'promo-tag-font-color',
            __('Promo tag font', 'ohbe'),
            array($this, 'promoTagFontColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Promo tag background color
        add_settings_field(
            'promo-tag-bg-color',
            __('Promo tag background', 'ohbe'),
            array($this, 'promoTagBgColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Best price tag font color
        add_settings_field(
            'best-price-tag-font-color',
            __('Best price tag font', 'ohbe'),
            array($this, 'bestPriceTagFontColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Best price tag background color
        add_settings_field(
            'best-price-tag-bg-color',
            __('Best price tag background', 'ohbe'),
            array($this, 'bestPriceTagBgColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Only left tag font color
        add_settings_field(
            'only-left-tag-font-color',
            __('Only left tag font', 'ohbe'),
            array($this, 'onlyLeftTagFontColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
        // Only left tag background color
        add_settings_field(
            'only-left-tag-bg-color',
            __('Only left tag background', 'ohbe'),
            array($this, 'onlyLeftTagBgColorCallback'),
            'oscar-hotel-settings',
            'color-settings'
        );
    }

    /**
     * Define settings section: Main Settings.
     *
     * @return void
     */
    public function mainSettingsCallback() {
        printf(
            '<p>%s</p>',
            __('Check out the Help section for '
            .'more information about the plugin.',
            'ohbe'
        ));
    }

    /**
     * Display client_code field in settings page.
     *
     * @return void
     */
    public function clientCodeCallback() {
        $html = sprintf(
            '<input type=text name=api_code id=ohbe_api_code value="%s">',
            OHBE_Main::getSettings('api_code')
        );

        $html .= sprintf(
            '<p>'
            . __('You can find your connection code on the '
            . '<a %s>booking engine integration page</a> in Oscar.', 'ohbe')
            . '</p>',
            'href=https://oscar.redforts.com/en/setup,be,integration,wordpress '
            . 'target=_blank'
        );
        echo $html;
    }

    /**
     * Display test_mode field in settings page.
     *
     * @return void
     */
    public function testModeCallback() {
        $value = OHBE_Main::getSettings('test_mode');
        printf(
            '<input type=checkbox name=test_mode id=ohbe_test_mode %s%s><p>%s</p>',
            $value,
            $value ? ' checked' : '',
            __('Test your booking engine with the Test mode on.', 'ohbe')
        );
    }

    /**
     * Display notes field in settings page.
     *
     * @return void
     */
    public function notesCallback() {
        $value = OHBE_Main::getSettings('notes');
        printf(
            '<input type=checkbox name=notes id=ohbe_notes %s%s>',
            $value,
            $value ? ' checked' : ''
        );
    }

    /**
     * Display conditions field in settings page.
     *
     * @return void
     */
    public function conditionsCallback() {
        $value = OHBE_Main::getSettings('conditions');
        printf(
            '<input type=checkbox name=conditions id=ohbe_conditions value="%s"%s>',
            $value,
            $value ? ' checked' : ''
        );
    }

    /**
     * Define help tabs.
     *
     * @return void
     */
    public static function addHelpTabs() {
        $screen = get_current_screen();
        $screen->add_help_tab(array(
            'id' => 'main_help',
            'title' => __('Overview', 'ohbe'),
            'content' => '',
            'callback' => array(__CLASS__, 'mainHelpCallback'),
        ));
        $screen->add_help_tab(array(
            'id' => 'form_help',
            'title' => __('Availability form', 'ohbe'),
            'content' => '',
            'callback' => array(__CLASS__, 'formHelpCallback'),
        ));
        $help_side_bar = sprintf(
            '<p><strong>%s</strong></p><p>'
            . '<a href=https://oscar.redforts.com/en/setup,be,integration,wordpress '
            . 'target=_blank>%s</a></p>',
            __('More info', 'ohbe'),
            __('Oscar & Wordpress', 'ohbe')
        );
        $screen->set_help_sidebar($help_side_bar);
    }

    /**
     * Define help callback.
     *
     * @return void
     */
    public static function helpCallback($section, $lang) {
        $help_file = sprintf('includes/admin/help/%s-help-%s.php', $section, $lang);
        if (file_exists(OHBE_PATH . $help_file)) {
            include OHBE_PATH . $help_file;
        }
        else {
            include OHBE_PATH . sprintf(
                'includes/admin/help/%s-help-en.php',
                $section
                );
        }
    }

    /**
     * Define main help callback.
     *
     * @return void
     */
    public static function mainHelpCallback() {
        self::helpCallback('main', self::getCurrentLocale());
    }

    /**
     * Define form help callback.
     *
     * @return void
     */
    public static function formHelpCallback() {
        self::helpCallback('form', self::getCurrentLocale());
    }

    /**
     * Display "only left" quantity field in settings page.
     *
     * @return void
     */
    public static function onlyLeftCallback() {
        printf(
            '<input type=number name=only_left id=ohbe_only_left value="%s" min=0>'
            . '<p>%s.</p>',
            OHBE_Main::getSettings('only_left'),
            __(
                'Show "Only left" tag when availability is '
                . 'less than or equal to this amount (0 = never)'
                , 'ohbe'
            )
        );
    }

    /**
     * Display "button" font color field in settings page.
     *
     * @return void
     */
    public static function btnFontColorCallback() {
        printf(
            '<input class=color-field name=btn_font_color value="%s">',
            OHBE_Main::getSettings('btn_font_color')
        );
    }

    /**
     * Display "button" background color field in settings page.
     *
     * @return void
     */
    public static function btnBgColorCallback() {
        printf(
            '<input class=color-field name=btn_bg_color value="%s">',
            OHBE_Main::getSettings('btn_bg_color')
        );
    }

    /**
     * Display "promo tag" font color field in settings page.
     *
     * @return void
     */
    public static function promoTagFontColorCallback() {
        printf(
            '<input class=color-field name=promo_tag_font_color value="%s">',
            OHBE_Main::getSettings('promo_tag_font_color')
        );
    }

    /**
     * Display "promo tag" background color field in settings page.
     *
     * @return void
     */
    public static function promoTagBgColorCallback() {
        printf(
            '<input class=color-field name=promo_tag_bg_color value="%s">',
            OHBE_Main::getSettings('promo_tag_bg_color')
        );
    }

    /**
     * Display "best price" tag font color field in settings page.
     *
     * @return void
     */
    public static function bestPriceTagFontColorCallback() {
        printf(
            '<input class=color-field name=best_price_tag_font_color value="%s">',
            OHBE_Main::getSettings('best_price_tag_font_color')
        );
    }

    /**
     * Display "best price" tag background color field in settings page.
     *
     * @return void
     */
    public static function bestPriceTagBgColorCallback() {
        printf(
            '<input class=color-field name=best_price_tag_bg_color value="%s">',
            OHBE_Main::getSettings('best_price_tag_bg_color')
        );
    }

    /**
     * Display "only left" tag font color field in settings page.
     *
     * @return void
     */
    public static function onlyLeftTagFontColorCallback() {
        printf(
            '<input class=color-field name=only_left_tag_font_color value="%s">',
            OHBE_Main::getSettings('only_left_tag_font_color')
        );
    }

    /**
     * Display "only left" tag background color field in settings page.
     *
     * @return void
     */
    public static function onlyLeftTagBgColorCallback() {
        printf(
            '<input class=color-field name=only_left_tag_bg_color value="%s">',
            OHBE_Main::getSettings('only_left_tag_bg_color')
        );
    }

    /**
     * Handle fields when form is submitted.
     *
     * @return void
     */
    public function saveSettings() {
        if (
            $_SERVER['REQUEST_METHOD'] == 'POST'
            && isset($_POST['update_ohbe_settings'])
            && check_admin_referer('update_ohbe_settings', 'update_ohbe_settings')
            && current_user_can('administrator')
        ) {
            OHBE_Main::setSettings(
                'api_code',
                isset($_POST['api_code']) ? $_POST['api_code'] : ''
            );
            OHBE_Main::setSettings(
                'test_mode',
                isset($_POST['test_mode']) ? 1 : 0
            );
            OHBE_Main::setSettings(
                'notes',
                isset($_POST['notes']) ? 1 : 0
            );
            OHBE_Main::setSettings(
                'conditions',
                isset($_POST['conditions']) ? 1 : 0
            );
            OHBE_Main::setSettings(
                'only_left',
                isset($_POST['only_left']) ? intval($_POST['only_left']) : 5
            );
            OHBE_Main::setSettings(
                'btn_font_color',
                isset($_POST['btn_font_color'])
                    ? $_POST['btn_font_color']
                    : '#fff'
            );
            OHBE_Main::setSettings(
                'btn_bg_color',
                isset($_POST['btn_bg_color'])
                    ? $_POST['btn_bg_color']
                    : '#000'
            );
            OHBE_Main::setSettings(
                'promo_tag_font_color',
                isset($_POST['promo_tag_font_color'])
                    ? $_POST['promo_tag_font_color']
                    : '#fff'
            );
            OHBE_Main::setSettings(
                'promo_tag_bg_color',
                isset($_POST['promo_tag_bg_color'])
                    ? $_POST['promo_tag_bg_color']
                    : '#279eff'
            );
            OHBE_Main::setSettings(
                'best_price_tag_font_color',
                isset($_POST['best_price_tag_font_color'])
                    ? $_POST['best_price_tag_font_color']
                    : '#fff'
            );
            OHBE_Main::setSettings(
                'best_price_tag_bg_color',
                isset($_POST['best_price_tag_bg_color'])
                    ? $_POST['best_price_tag_bg_color']
                    : '#11a005'
            );
            OHBE_Main::setSettings(
                'only_left_tag_font_color',
                isset($_POST['only_left_tag_font_color'])
                    ? $_POST['only_left_tag_font_color']
                    : '#fff'
            );
            OHBE_Main::setSettings(
                'only_left_tag_bg_color',
                isset($_POST['only_left_tag_bg_color'])
                    ? $_POST['only_left_tag_bg_color']
                    : '#ff6767'
            );
            OscarAPI::getInventoryData();
        }
    }

    /**
     * Enqueue CSS and JS.
     *
     * @return void
     */
    public function loadAssets() {
        // JS
        wp_enqueue_script(
            'ohbe-admin', OHBE_URL.'assets/js/ohbe-admin.js',
            array('jquery', 'wp-color-picker' ), OHBE_VERSION, 1
        );
        // CSS
        wp_enqueue_style('wp-color-picker');
    }

    /**
     * Sanitize user input for options.
     *
     * @return void
     */
    public function sanitizeInput($input) {
        $valid_input = array();
        // Connection_code
        $connection_code = trim($input['api_code']);
        if (preg_match('/^[0-9a-zA-Z]{8,}+$/', $connection_code)) {
            $valid_input['connection_code'] = strip_tags(
                stripslashes($connection_code)
            );
        }
        elseif (empty($connection_code)) {
            $valid_input['connection_code'] = '';
        }
        else {
            add_settings_error(
                'oscar-hotel-settings',
                'ohbe-connection-code-error',
                __('Not a correct connection code', 'ohbe'),
                'error'
            );
            $valid_input['connection_code'] = $this->settings['connection_code'];
        }
        // Test_mode
        $valid_input['test_mode'] = isset($input['test_mode']) ? 1 : 0;
        // Notes
        $valid_input['notes'] = isset($input['notes']) ? 1 : 0;
        // Conditions
        $valid_input['conditions'] = isset($input['conditions']) ? 1 : 0;

        apply_filters('sanitize_options', $valid_input, $input);
    }

    /**
     * Get current locale
     *
     * @return string Current locale. Example: 'en'
     */
    public static function getCurrentLocale() {
        return substr(get_locale(), 0, 2);
    }
}
