<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * API class
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */

class OscarAPI
{
    const API_VERSION = 3;
    const BASE_URI = 'https://booking.redforts.com/wpp/';

    /**
     * Get the available accommodations in Oscar.
     *
     * @param string $arrival Arrival date. Format: Y-m-d.
     * @param string $departure Departure date. Format: Y-m-d.
     * @param string $promo_code Promotion code.
     * @return array Oscar availability call response.
     */
    public static function getAvailability($arrival, $departure, $promo_code="") {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
                "arrival" => $arrival,
                "departure" => $departure,
                "lang" => self::getLanguage(),
                "promo" => $promo_code,
                "system_info" => array(
                    "ohbe" => OHBE_VERSION,
                    "os" => php_uname('s') . ' ' . php_uname('r'),
                    "php" => phpversion(),
                    "wordpress" => get_bloginfo('version'
                ),
            ));

            return OscarAPI::send('availability', $body);
        }
    }

    /**
     * Get the calendar dates availability in Oscar.
     *
     * @return array Oscar calendar data call response.
     */
    public static function getCalendarData() {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
                );

            return OscarAPI::send('calendar_data', $body);
        }
    }

    /**
     * Get the inventory in Oscar.
     *
     * @return array Oscar inventory call response.
     */
    public static function getInventory() {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                "api_version" => self::API_VERSION,
            );

            return OscarAPI::send('inventory', $body);
        }
    }

    /**
     * Get the available item list in Oscar.
     *
     * @return array Oscar item list data call response.
     */
    public static function getItemList() {
        if (OHBE_Main::getSettings('api_code')) {
            extract(OHBE_Reservation::getReservationParam());
            $body = array(
                "api_version" => self::API_VERSION,
                "arrival" => $arrival->format('Y-m-d'),
                "lang" => self::getLanguage(),
                "reserv_id" => intval($reserv_id),
                "email" => $email,
            );

            return OscarAPI::send('item_list', $body);
        }
    }

    /**
     * Update inventory data from Oscar.
     *
     * @return void
     */
    public static function getInventoryData() {
        $data = OscarAPI::getInventory();

        if (is_array($data) && isset($data['version'])) {
            $ohbe_inventory = array(
                'version' => isset($data['version']) ? $data['version'] : 0,
                'accos' => isset($data['accos']) ? $data['accos'] : array(),
                'languages' => isset($data['languages'])
                    ? $data['languages'] : array(),
                'fields' => isset($data['fields']) ? $data['fields'] : array(),
                'rates' => isset($data['rates']) ? $data['rates'] : array(),
                'issuers' => isset($data['creditcard_issuers'])
                    ? $data['creditcard_issuers'] : array(),
                'currency' => isset($data['currency']) ? $data['currency'] : array(),
            );

            foreach ($data['fields'] as $field) {
                if ($field['id'] == 'ohbeSurname2') {
                    $ohbe_inventory['surnames'] = true;
                }
            }

            update_option('ohbe_inventory', $ohbe_inventory);
        }
    }

    /**
     * Set a language for API calls to Oscar based on WP locale.
     *
     * @return string|null Locale slug. Example: 'en'
     */
    public static function getLanguage() {
        $languages = OHBE_Main::getInventory('languages');

        if ($languages) {
            foreach ($languages as $language) {
                // Polylang
                if(
                    function_exists('pll_current_language')
                    && $language == pll_current_language()
                ) {
                    return pll_current_language();
                }
                // Locale
                if (
                    $language == substr(get_locale(), 0, 2)
                    || $language == get_locale()
                ) {
                    return $language;
                }
            }
        }

        return null;
    }

    /**
     * Get the full url for the API call.
     *
     * @param string $call API call.
     * @return string|null Locale slug. Example: 'en'
     */
    public static function getUrl($call) {
        return self::BASE_URI . OHBE_Main::getSettings('api_code') . '/' . $call;
    }

    /**
     * Update the item list for reservation in Oscar.
     *
     * @param array $item Item to update.
     * @return array Oscar reservation call response
     */
    public static function setItemsUpdate($item) {
        if (OHBE_Main::getSettings('api_code')) {
            extract(OHBE_Reservation::getReservationParam());
            $body = array(
                'api_version' => self::API_VERSION,
                'arrival' => $arrival->format('Y-m-d'),
                'lang' => self::getLanguage(),
                'reserv_id' => intval($reserv_id),
                'email' => $email,
                'items' => array($item)
            );

            return OscarAPI::send('items_update', $body);
        }
    }

    public static function send($call, $body) {
        return json_decode(
            wp_remote_retrieve_body(
                wp_remote_post(
                    OscarAPI::getUrl($call),
                    array(
                        'headers' => array(
                            'Content-Type' => 'application/json; charset=utf-8'
                        ),
                        'body' => json_encode($body),
                    )
                )
            ),
            true
        );
    }

    /**
     * Make a reservation in Oscar.
     *
     * @param string $arrival Arrival date. Format: Y-m-d.
     * @param string $departure Departure date. Format: Y-m-d.
     * @param string $status Status of the reservation.
     * @param array $accos Accommodations.
     * @param array $reservation_params Reservation params.
     * @param string $promo_code Promotion code.
     * @return array Oscar reservation call response
     */
    public static function setReservation(
        $arrival,
        $departure,
        $status,
        $accos,
        $reservation_params,
        $promo_code=''
    ) {
        if (OHBE_Main::getSettings('api_code')) {
            $body = array(
                'api_version' => self::API_VERSION,
                'api_test' => (OHBE_Main::getSettings('test_mode')
                    && OHBE_Main::getSettings('test_mode') == 1) ? true : false,
                'arrival' => $arrival,
                'departure' => $departure,
                'lang' => self::getLanguage(),
                'promo' => $promo_code,
                'status' => $status,
                'accos' => $accos,
            );
            foreach ($reservation_params as $key => $value) {
                $body[substr($key, 5)] = $value;
            }

            return OscarAPI::send('reservation', $body);
        }
    }
}
