<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Availability functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Availability
{
    /**
     * Handle accommodation availability errors if set.
     *
     * @return object|null Error object or null.
     */
    public static function accommodationAvailabilityErrors($type, $filters=null) {
        $errors = new WP_Error();

        switch($type) {
            case 'no_filters':
                $errors->add(
                    'no_filters',
                    __('Please introduce the arrival and departure dates', 'ohbe')
                );
                break;
            case 'invalid_param':
                $errors->add(
                    'invalid_param',
                    __('Invalid search parameter', 'ohbe')
                );
            case 'position':
                if (
                    $filters['ci']->format('Y-m-d')
                    >= $filters['co']->format('Y-m-d')
                ) {
                    $errors->add(
                        'ohbe_wrong_date_position',
                        __('Departure not after arrival.', 'ohbe')
                    );
                }
            default:
                return null;
        }

        return $errors;
    }

    /**
     * Create the accommodation array structure.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $all_filters Filters set.
     * @return array Accommodations structure.
     */
    public static function accommodationsHandle($redforts_data, $all_filters) {
        $accos = array();
        if (isset($redforts_data['options'])) {
            foreach ($redforts_data['options'] as $options) {
                $acco_arrival = DateTime::createFromFormat(
                    '!Y-m-d',
                    $options['arrival']
                );
                $acco_departure = DateTime::createFromFormat(
                    '!Y-m-d',
                    $options['departure']
                );
                $range_type = 'selected';

                if (
                    $acco_arrival > $all_filters['ci']
                    || $acco_departure < $all_filters['co']
                ) {
                    $range_type = 'limited';
                }
                if (
                    $acco_arrival < $all_filters['ci']
                    || $acco_departure > $all_filters['co']
                ) {
                    $range_type = 'suggested';
                }
                if (
                    (isset($_GET['acco']) && $options['acco_id'] == $_GET['acco'])
                    || !isset($_GET['acco'])
                ) {
                    $promo_code = isset($all_filters['pr'])
                        ? $all_filters['pr'] : null;
                    self::filterAccommodations(
                        $options,
                        $accos,
                        $range_type,
                        $promo_code
                    );
                }
            }
        }

        return $accos;
    }

    /**
     * Add clearfix to accommodation card for grid.
     *
     * @param int $count
     * @return void
     */
    public static function printClearfix($count) {
        if ($count % 2 == 0) {
            echo '<li class="clearfix visible-sm-block hidden-md hidden-lg"></li>';
        }
        if ($count % 3 == 0) {
            echo '<li class="clearfix hidden-sm visible-md-block visible-lg-block">'
                . '</li>';
        }
    }

    /**
     * Create period structure for an accommodation.
     *
     * @param array $options Accommodations available in Oscar.
     * @param string $range_type Range of days available
     *      ("selected", "limited", "suggested").
     * @param array $accos Accommodations structure.
     * @param string $promo_code Promo code if set.
     * @param bool $new_acco Tell if the accommodation is set or not.
     * @return void
     */
    public static function createPeriods(
        $options,
        $range_type,
        &$accos,
        $promo_code,
        &$new_acco=false
    ) {
        $count = 0;
        $acco_id = $options['acco_id'];
        $price = (string)$options['price'];
        $range = $options['arrival'] . '_' . $options['departure'];
        $p = 'periods';
        if ($new_acco) {
            foreach ($options['periods'] as $period) {
                $period['arrival'] = $options['arrival'];
                $period['departure'] = $options['departure'];
                $period['adults'] = $options['adults'];
                $period['children'] = $options['children'];
                $period['excl_taxes'] = $options['excl_taxes'];
                $period['night_string'] = sprintf(
                    _n('%s night', '%s nights', $period['days'], 'ohbe'),
                    $period['days']
                );
                if (isset($options['orig_price']) && !$new_acco['promo_code']) {
                    $new_acco['promo_code'] = $promo_code;
                }
                if ($count == 0) {
                    $period['sorted'] = 0;
                    $new_acco[$p][$price][][] = $period;
                }
                else {
                    $sorted = count($new_acco[$p][$price]) - 1;
                    $period['sorted'] = $sorted;
                    $new_acco[$p][$price][$sorted][] = $period;
                }
                $count++;
            }
            $accos[$range_type][$range][$acco_id] = $new_acco;
        }
        else {
            foreach ($options['periods'] as $period) {
                $period['arrival'] = $options['arrival'];
                $period['departure'] = $options['departure'];
                $period['adults'] = $options['adults'];
                $period['children'] = $options['children'];
                $period['excl_taxes'] = $options['excl_taxes'];
                $period['night_string'] = sprintf(
                    _n('%s night', '%s nights', $period['days'], 'ohbe'),
                    $period['days']
                );
                if (
                    isset($options['orig_price'])
                    && !isset($accos[$range_type][$range][$acco_id]['promo_code'])
                ) {
                    $accos[$range_type][$range][$acco_id]['promo_code'] = $promo_code;
                }
                if ($count == 0) {
                    $sorted = isset(
                        $accos[$range_type][$range][$acco_id][$p][$price]
                    )
                        ? count($accos[$range_type][$range][$acco_id][$p][$price])
                        : 0;
                    $period['sorted'] = $sorted;
                    $accos[$range_type][$range][$acco_id][$p][$price][][] = $period;
                }
                else {
                    $sorted = count(
                        $accos[$range_type][$range][$acco_id][$p][$price]
                    ) - 1;
                    $period['sorted'] = $sorted;
                    $accos[$range_type][$range][$acco_id][$p]
                        [$price][$sorted][] = $period;
                }
                $count++;
            }
            // Set the accommodation lowest price/orig_price by default
            if ($price < $accos[$range_type][$range][$acco_id]['price']) {
                $accos[$range_type][$range][$acco_id]['price'] = $price;
            }
        }
    }

    /**
     * @param array $options Accommodations available in Oscar.
     * @param string $range_type Range of days available
     *      ("selected", "limited", "suggested").
     * @param array $accos Accommodations structure.
     * @param string $promo_code Promo code if set.
     * @return void
     */
    public static function filterAccommodations(
        $options,
        &$accos,
        $range_type,
        $promo_code=null
    ) {
        $range = $options['arrival'] . '_' . $options['departure'];

        if (!isset($accos[$range_type][$range][$options['acco_id']])) {
            $nights_number = OHBE_Tools::getNightsInterval($options);
            $new_acco = array(
                'acco_id' => $options['acco_id'],
                'arrival' => $options['arrival'],
                'departure' => $options['departure'],
                'price' => $options['price'],
                'amount' => $options['amount'],
                'adults' => $options['adults'],
                'children' => $options['children'],
                'excl_taxes' => $options['excl_taxes'],
                'date_range' => $range_type,
                'promo_code' => null,
                'nights' => $nights_number,
                'night_string' => sprintf(
                    _n('%s night', '%s nights', $nights_number, 'ohbe'),
                    $nights_number
                ),
                'periods' => array(),
            );
            if (isset($options['name'])) {
                $new_acco['name'] = $options['name'];
            }
            if (isset($options['orig_price'])) {
                $new_acco['orig_price'] = $options['orig_price'];
            }
            self::createPeriods(
                $options,
                $range_type,
                $accos,
                $promo_code,
                $new_acco
            );
        }
        else{
            self::createPeriods($options, $range_type, $accos, $promo_code);
        }
    }

    /**
     * Get accommodations panel printed in the page.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $all_filters Filters set.
     * @return array|null
     */
    public static function getAccommodationsStructure($redforts_data, $all_filters) {
        if (!is_null(ohbeAvailabilityFormHandle()) || is_null($redforts_data)) {
            return null;
        }
        $updated_oscar_data = OHBE_Tools::checkVersion(
            $redforts_data['inventory_version']
        );
        if ($updated_oscar_data) {
            OscarAPI::getInventoryData();
        }
        return OHBE_Availability::accommodationsHandle(
            $redforts_data,
            $all_filters
        );
    }

    /**
     * Print accommodation card's attributes.
     *
     * @param  array $accos Accommodations structure.
     * @param  array $acco Single accommodation structure.
     * @return void
     */
    public static function printAccommodationCardData($accos, $acco) {
        printf(
            'data-name="%s" data-range=%s data-id=%s data-amount=%s '
            . 'data-arrival=%s data-departure=%s data-price=%s data-taxes="%s" '
            . 'data-nights=%s data-night-string="%s" data-rate=%s '
            . 'data-sorted=%s %s %s',
            $accos[$acco['acco_id']]['name'],
            $acco['date_range'],
            $acco['acco_id'],
            $acco['amount'],
            $acco['arrival'],
            $acco['departure'],
            $acco['price'],
            $acco['excl_taxes'],
            $acco['nights'],
            $acco['night_string'],
            OHBE_Availability::renderDefaultRatePartIds($acco['periods']),
            "0",
            OHBE_Availability::renderDefaultAccommodationRateAdultsAndChildren(
                $acco['periods']
            ),
            isset($acco['promo_code'])
                ? sprintf('data-promo-code=%s', $acco['promo_code']) : ''
        );
    }

    /**
     * Print available accommodations.
     *
     * @param array $accommodations Accommodations structure.
     * @param array $redforts_data Oscar availability call response.
     * @param array $all_filters Filters set.
     * @return void
     */
    public static function printAccommodationLists(
        $accommodations,
        $redforts_data,
        $all_filters
    ) {
        // These values are used in accommodation_card.php
        $rates = isset($redforts_data['rates']) ? $redforts_data['rates'] : null;
        $accos = isset($redforts_data['accos']) ? $redforts_data['accos'] : null;
        $promo_code = isset($all_filters['pr']) ? $all_filters['pr'] : null;

        if (
            $accommodations
            && count($accommodations) > 0
            && !isset($accommodations['selected'])
        ) {
            printf(
                '<div class=ohbe-noaccos><span>%s</span><span>%s</span></div>',
                __('There are no accommodations available for the selected dates.',
                    'ohbe'),
                __('Check out the options below or try another period.', 'ohbe')
            );
        }
        if ($accommodations) {
            // JS
            wp_enqueue_script('jquery-ui-accordion');
            wp_enqueue_script('ohbe_moment');
            wp_enqueue_script('ohbe_availability');
            wp_localize_script(
                'ohbe_availability',
                'accommodations',
                $accommodations
            );
        }
        if (isset($accommodations['selected'])) {
            usort(
                $accommodations['selected']
                [current(array_keys($accommodations['selected']))],
                array('OHBE_Tools','sortByPrice')
            );
            printf(
                '<div class=availability-header>'
                . '<h3 class=acco-list-title>%s</h3>'
                . '</div>',
                __('Select accommodations', 'ohbe')
            );
            foreach ($accommodations['selected'] as $acco_date) {
                print('<ul class="accommodation-list row">');
                $count = 1;
                foreach ($acco_date as $acco) {
                    ksort($acco['periods']);
                    include(
                        OHBE_PATH . 'includes/templates/views/accommodation_card.php'
                    );
                    OHBE_Availability::printClearfix($count);
                    $count++;
                }
                print('</ul>');
            }
        }
        if (isset($accommodations['limited'])) {
            foreach ($accommodations['limited'] as $key => $value) {
                usort(
                    $accommodations['limited'][$key],
                    array('OHBE_Tools','sortByPrice')
                );
                $dates = explode('_', $key);
                if (isset($dates[0]) && isset($dates[1])) {
                    $arrival = DateTime::createFromFormat('Y-m-d', $dates[0]);
                    $departure = DateTime::createFromFormat('Y-m-d', $dates[1]);
                    printf(
                        '<div class="alert alert-success">%s: %s - %s</div>
                        <ul class="accommodation-list row">',
                        __('Limited availability', 'ohbe'),
                        $arrival->format('d/m/Y'),
                        $departure->format('d/m/Y')
                    );
                    $count = 1;
                    foreach ($value as $acco) {
                        ksort($acco['periods']);
                        include(
                            OHBE_PATH . 'includes/templates/'
                            . 'views/accommodation_card.php'
                        );
                        OHBE_Availability::printClearfix($count);
                        $count++;
                    }
                    print('</ul>');
                }
            }
        }
        if (isset($accommodations['suggested'])) {
            foreach ($accommodations['suggested'] as $key => $value) {
                usort(
                    $accommodations['suggested'][$key],
                    array('OHBE_Tools','sortByPrice')
                );
                $dates = explode('_', $key);
                if (isset($dates[0]) && isset($dates[1])) {
                    $arrival = DateTime::createFromFormat('Y-m-d', $dates[0]);
                    $departure = DateTime::createFromFormat('Y-m-d', $dates[1]);
                    printf(
                        '<div class=ohbe-accordion>'
                        . '<div class="alert alert-availability">'
                        . '<span class=range>%s</span> '
                        . '<span class=dates>%s - %s</span></div>'
                        . '<ul class="accommodation-list row">',
                        __('Offers for other dates', 'ohbe'),
                        $arrival->format('d/m/Y'),
                        $departure->format('d/m/Y')
                    );
                    $count = 1;
                    foreach ($value as $acco) {
                        ksort($acco['periods']);
                        include(
                            OHBE_PATH . 'includes/templates/views/'
                            . 'accommodation_card.php'
                        );
                        OHBE_Availability::printClearfix($count);
                        $count++;
                    }
                    print('</ul></div>');
                }
            }
        }
    }

    /**
     * Print error alert(s) when exist.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return void
     */
    public static function printErrors($redforts_data) {
        if (isset($redforts_data['errors'])) {
            foreach ($redforts_data['errors'] as $error) {
                printf(
                    '<div class="alert alert-danger">%s</div>',
                    htmlspecialchars($error['text'])
                );
            }
        }
    }

    /**
     * Print alert when there are no accommodations available.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param array $accos Accommodations structure.
     * @return void
     */
    public static function printNoAccommodationsAvailable($redforts_data, $accos) {
        if (
            !isset($redforts_data['errors'])
            && (empty($redforts_data['options']) || empty($accos))
        ) {
            printf(
                '<div class="alert alert-danger">'
                . '<i class="fa fa-exclamation-triangle" aria-hidden=true></i>'
                . ' <span>%s</span> <span>%s</span></div>',
                __('There are no accommodations available for the selected dates.',
                    'ohbe'),
                __('Please try another period.', 'ohbe')
            );
        }
    }

    /**
     * Print available accommodations previously selected.
     *
     * @param array $redforts_data Oscar availability call response.
     * @param int $total_price Selected accommodations' total price.
     * @return void
     */
    public static function printSelectedAccommodations(
        $redforts_data,
        &$total_price
    ) {
        $rates = $redforts_data['rates'];

        if (isset($_SESSION['ohbe']['selected_accos'])) {
            $isRemovable = true;
            foreach ($_SESSION['ohbe']['selected_accos'] as $acco) {
                // Comparisson between selected accos and accos loaded by Oscar is
                // needed in order to known if the accommodation is still available.
                foreach ($redforts_data['options'] as $option) {
                    if (
                        $acco['acco_id'] == $option['acco_id']
                        && $acco['adults'] == $option['adults']
                        && $acco['arrival'] == htmlspecialchars($option['arrival'])
                        && $acco['children'] == $option['children']
                        && $acco['departure'] == $option['departure']
                        && $acco['excl_taxes'] == $option['excl_taxes']
                        && $acco['price'] == $option['price']
                        && $acco['quantity'] <= $option['amount']
                    ) {
                        $rate_ids = '';
                        for ($i=0; $i<count($option['periods']); $i++) {
                            if ($i > 0) {
                                $rate_ids .= ',';
                            }
                            $rate_ids .= $option['periods'][$i]['rate_id'];
                        }
                        if ($acco['rate'] == $rate_ids) {
                            $total_price += $acco['quantity'] * $acco['price'];
                            include(
                                OHBE_PATH . 'includes/templates/views/'
                                . 'accommodation_reservation_card.php'
                            );
                        }
                    }
                }
            }
        }
    }

    /**
     * Render accommodation card's accos-total block.
     *
     * @param  array $acco Single accommodation structure.
     * @param bool $default_rate_has_promo Tell if the default rate has promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderAccosTotalBlock($acco, $default_rate_has_promo) {
        $select = '';
        $only_left = '';
        $orig_price = 0;
        $price = 0;
        $s_acco = OHBE_Tools::getSelectedAcco($acco);

        if ($s_acco && isset($acco['promo_code'])) {
            $orig_price = $acco['orig_price'] * $s_acco['quantity'];
            $price = $s_acco['price'] * $s_acco['quantity'];
        }
        else if ($s_acco) {
            $orig_price = $acco['price'] * $s_acco['quantity'];
        }

        for ($i=0; $i<=$acco['amount']; $i++) {
            $select .= sprintf(
                '<option value=%1$s%2$s>%1$s</option>',
                $i,
                $s_acco && $s_acco['quantity'] == $i ? ' selected' : ''
            );
        }
        if ($acco['amount'] <= OHBE_Main::getSettings('only_left')) {
            $only_left = sprintf(
                '<div class=only-left>%s</div>',
                sprintf(__('Only %s left', 'ohbe'), $acco['amount'])
            );
        }
        $html = sprintf(
            '<div class=quantity-total-block>'
            . '<div class=accos-quantity>'
            . '<div class=ohbe-title>%s:</div>'
            . '<div class=value-wrapper>'
            . '%s<select class=value>%s</select></div></div>'
            . '<div class=total%s>'
            . '<div class=ohbe-title>%s:</div>'
            . '<div class="value%s">%s</div>'
            . '</div>',
            __('Accommodations', 'ohbe'),
            $only_left,
            $select,
            is_null($s_acco) ? ' style="display:none;"' : '',
            $acco['excl_taxes']
                ? sprintf(
                    esc_html(__('Total excluding %1$s%% taxes', 'ohbe')),
                    $acco['excl_taxes']
                )
                : __('Total including taxes', 'ohbe'),
            $default_rate_has_promo ? ' orig' : '',
            OHBE_Tools::formatPrice($orig_price)
        );
        $html .= sprintf(
            '<div class="total-promo"%s>'
            . '<div class="ohbe-title tag">%s</div>'
            . '<div class="value%s">%s</div>'
            . '</div>',
            $s_acco && isset($acco['orig_price'])
                ? ''
                : ' style="display:none;"',
            $acco['promo_code'],
            $default_rate_has_promo ? ' orig' : '',
            OHBE_Tools::formatPrice($price)
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate info block.
     *
     * @param  array $acco Single accommodation structure.
     * @param  array $selected_acco Single selected accommodation structure.
     * @param array $rates Rates info from Oscar availability call.
     * @param string $first_price_key First key of periods structure.
     * @param string $total_rate_orig_price Total rate original price.
     * @param bool $default_rate_has_promo Tell if the default rate has promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderCombinedRateInfo(
        $acco,
        $selected_acco,
        $rates,
        $first_price_key,
        &$total_rate_orig_price,
        &$default_rate_has_promo
    ) {

        if ($selected_acco) {
            // This accommodation has been selected previously
            $periods = $selected_acco['periods'];
            $sorted = $selected_acco['sorted'];
        }
        else {
            // This accommodation hasn't been selected previously
            $periods = $acco['periods'][$first_price_key][0];
            $sorted = 0;
        }
        $rates_combined_list = '';
        $total_rate_orig_price += isset($acco['orig_price'])
            ? $acco['orig_price']
            : $acco['price'];
        foreach ($periods as $period_part) {
            if (isset($period_part['orig_price'])) {
                $default_rate_has_promo = true;
            }
            $rates_combined_list .= sprintf(
                '<li>%s %s (%s)</li>',
                OHBE_Tools::renderOccupancyIcons(
                    $period_part['rate_id'],
                    'rates'
                ),
                $rates[$period_part['rate_id']]['name'],
                sprintf(
                    _n('%s night', '%s nights', $period_part['days'], 'ohbe'),
                    $period_part['days']
                )
            );
        }
        if ($default_rate_has_promo) {
            $data_prices = sprintf(
                'data-price=%s data-orig-price=%s',
                $acco['price'],
                $acco['orig_price']
            );
        }
        else {
            $data_prices = sprintf(
                'data-price=%s',
                $acco['price']
            );
        }

        return sprintf(
            '<div class=info data-amount=%s data-sorted=%s %s>'
            . '<div class=name>%s</div>'
            . '<div class="conditions modal-button" '
            . 'role=button data-modal=conditions-modal>'
            . '<a><span class="dashicons dashicons-info"></span> %s</a></div>'
            . '<ul class=combined-rates>%s</ul>',
            $acco['amount'],
            $sorted,
            $data_prices,
            __('Combination of rates', 'ohbe'),
            __('Conditions', 'ohbe'),
            $rates_combined_list
        );
    }

    /**
     * Render accommodation card's combined rate list block.
     *
     * @param  int $nights Nights.
     * @param  array $period_parts Rate period parts.
     * @param string $first_price_key First key of periods structure.
     * @param array $rates Rates info from Oscar availability call.
     * @param string $rate_price Rate price.
     * @param string $sorted Order of the rate (used when there are more than one
     * with the same price).
     * @param string $best_price_tag Best price tag's html structure
     * @param string $total_rate_orig_price Total rate original price.
     * @param string $full_rate_structure Empty string to be filled and get all
     * rate ids in a row
     * @param bool $rate_has_promo Tell if the rate has promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderCombinedRateList(
        $nights,
        $period_parts,
        $first_price_key,
        $rates,
        $rate_price,
        $sorted,
        $best_price_tag,
        &$total_rate_orig_price,
        &$full_rate_structure,
        &$rate_has_promo
    ) {
        $rate_part = '';
        for ($i=0; $i<count($period_parts); $i++) {
            if ($i > 0) {
                $full_rate_structure .= ',';
            }
            $full_rate_structure .= $period_parts[$i]['rate_id'];
            $rate_has_promo = isset($rate['orig_price']);
            $total_rate_orig_price +=
                isset($period_parts[$i]['orig_price'])
                    ? $period_parts[$i]['orig_price']
                    : $period_parts[$i]['price'];
            $rate_part .= sprintf(
                '<li>%s (%s)</li>',
                $rates[$period_parts[$i]['rate_id']]['name'],
                $period_parts[$i]['night_string']
            );
        }

        return sprintf(
            '<div class=ohbe-title>%s %s</div>'
            . '<div class="value%s">%s %s</div>'
            . '<ul>%s</ul>',
            OHBE_Tools::renderOccupancyIcons(
                $period_parts[0]['rate_id'],
                'rates'
            ),
            __('Combination of rates', 'ohbe'),
            isset($rate['orig_price']) ? ' orig' : '',
            !$rate_has_promo
            && $first_price_key === $rate_price
            && $sorted === 0 ? $best_price_tag : '',
            OHBE_Tools::formatPrice(
                $total_rate_orig_price / $nights
            ),
            $rate_part
        );
    }

    /**
     * Render the default accommodation rate adults & children data.
     * In other words, the adults & children data for a certain rate
     * when the page initializes.
     *
     * @param array $periods Accommodation periods
     * @return string
     */
    public static function renderDefaultAccommodationRateAdultsAndChildren(
        $periods
    ) {
        $first_price_key = current(array_keys($periods));

        return sprintf('data-adults="%s" data-children="%s"',
           $periods[$first_price_key][0][0]['adults'],
           $periods[$first_price_key][0][0]['children']
       );
    }

    /**
     * Render the default accommodation rate(s) ids when the page initializes.
     *
     * @param array $periods Accommodation periods
     * @return string Rate id(s). Example: 4,30,100.
     */
    public static function renderDefaultRatePartIds($periods) {
        $first_price_key = current(array_keys($periods));
        $html = '';
        for ($i=0; $i<count($periods[$first_price_key][0]); $i++) {
            if ($i > 0) {
                $html .= ',';
            }
            $html .= $periods[$first_price_key][0][$i]['rate_id'];
        }

        return $html;
    }

    /**
     * Render accommodation card's image.
     *
     * @param  array $accos Accommodations structure.
     * @param  array $acco Single accommodation structure.
     * @return string Image's html structure.
     */
    public static function renderAccommodationCardImage($accos, $acco) {
        $html = '';
        $html .= OHBE_Tools::renderAccoImage($acco['acco_id']);

        // Create link to modal
        if (
            isset($accos[$acco['acco_id']]['description'])
            || $html != ''
        ) {
            return sprintf(
                '<a class="description modal-button" '
                . 'role=button data-modal=description-modal>%s</a>',
                $html
            );
        }

        return $html;
    }

    /**
     * Render accommodation card's name.
     *
     * @param  array $accos Accommodations structure.
     * @param  array $acco Single accommodation structure.
     * @return string Name's html structure.
     */
    public static function renderAccommodationCardName($accos, $acco) {
        $name = $accos[$acco['acco_id']]['name'];
        if (isset($accos[$acco['acco_id']]['description'])) {
            return sprintf(
                '<a class="description modal-button" '
                . 'role=button data-modal=description-modal title=%s>'
                . '%s<span class="dashicons dashicons-info"></span>'
                . '</a>',
                __('Description', 'ohbe'),
                $name
            );
        }

        return $name;
    }

    /**
     * Render booking summary form url.
     *
     * @param string $page Page name
     * @return string Url with params if exist
     */
    public static function renderFormAction($page) {
        return sprintf('%s?%s%s%s',
            OHBE_Tools::getBaseUrl(),
            $page,
            isset($_GET['ohbe']) ? '&ohbe=' . urlencode($_GET['ohbe']) : '',
            isset($_GET['ispr']) ? '&ispr' : ''
        );
    }

    /**
     * Render modals.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return string Modals' html structure.
     */
    public static function renderModals($redforts_data) {
        // Description
        $html = sprintf(
            '<div id=ohbe-description-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%s</h4>'
            . '</div>'
            . '<div class=modal-body></div>'
            . '<div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-primary accept">%s</button>'
            . '</div></div></div></div>',
            __('Description', 'ohbe'),
            __('Close', 'ohbe')
        );
        // Conditions
        $html .= sprintf(
            '<div id=ohbe-conditions-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class="modal-title header">%s</h4>'
            . '</div>'
            . '<div class=modal-body>'
            . '<div class=conditions-content>',
            __('Reservation conditions', 'ohbe')
        );
        if (isset($redforts_data['checkinout'])) {
            $html .= sprintf(
                '<h4 class=header>%s</h4>'
                . '<p class=time-text>%s</p>',
                __('Check in/out conditions', 'ohbe'),
                nl2br(htmlspecialchars($redforts_data['checkinout']))
            );
        }
        if (isset($redforts_data['conditions'])) {
            $html .= sprintf(
                '<h4 class=header>%s</h4>'
                . '<p class=conditions-text>%s</p>',
                __('Conditions', 'ohbe'),
                nl2br(htmlspecialchars($redforts_data['conditions']))
            );
        }
        $html .= sprintf(
            '</div><div class=rate-content></div></div>'
            . '<div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-primary accept">%s</button>'
            . '</div></div></div></div>',
            __('Close', 'ohbe')
        );
        // Different dates
        $html .= sprintf(
            '<div id=ohbe-different-dates class=ohbe-modal>'
            . '<div class=modal-dialog role="document">'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%s</h4>'
            . '</div>'
            . '<div class=modal-body>%s</div>'
            . '<div class=modal-footer>'
            . '<button type=button id=ohbe-different-dates-yes '
            . 'class="btn-ohbe btn-other-dates">%s</button>'
            . '<button type=button id=ohbe-different-dates-no '
            . 'class="btn-ohbe btn-other-dates">%s</button>'
            . '</div></div></div></div>',
            __('Confirm', 'ohbe'),
            __('You have already selected accommodations for other dates. '
            . 'Do you want to discard them?', 'ohbe'),
            __('Yes', 'ohbe'),
            __('No', 'ohbe')
        );

        return $html;
    }

    /**
     * Print promo ribbon when there is some rate with promotion.
     *
     * @param array $acco Single accommodation structure.
     * @return void
     */
    public static function printPromoRibbon($acco) {
        if (isset($acco['promo_code'])) {
            printf(
                '<div class=ribbon>'
                . '<span>%s</span></div>',
                __('Promo', 'ohbe')
            );
        }
    }

    /**
     * Render accommodation card's rate block.
     *
     * @param array $acco Single accommodation structure.
     * @param array $rates Rates info from Oscar availability call.
     * @return string Rate block's html structure.
     */
    public static function renderRateBlock($acco, $rates) {
        $selected_acco = OHBE_Tools::getSelectedAcco($acco);
        $first_price_key = current(array_keys($acco['periods']));
        $total_rate_orig_price = 0;
        $default_rate_has_promo = false;
        $best_price_tag = sprintf(
            '<div class="tag best-price">%s</div>',
            __('Best price', 'ohbe')
        );
        $html = '<div class=rate-block>';

        // Rate info
        if (count($acco['periods'][$first_price_key][0]) == 1) {
            // UNCOMBINED RATE
            $html .= OHBE_Availability::renderUncombinedRateInfo(
                $acco,
                $selected_acco,
                $rates,
                $first_price_key,
                $total_rate_orig_price,
                $default_rate_has_promo
            );
        }
        else {
            // COMBINED RATES
            $html .= OHBE_Availability::renderCombinedRateInfo(
                $acco,
                $selected_acco,
                $rates,
                $first_price_key,
                $total_rate_orig_price,
                $default_rate_has_promo
            );
        }
        $html .= sprintf(
            '<div class=price>'
            . '<div class=ohbe-title>%s:</div>'
            . '<div class="value%s">%s<span>%s</span>'
            . '</div></div>',
            __('Per night', 'ohbe'),
            $default_rate_has_promo ? ' orig' : '',
            $default_rate_has_promo ? '' : $best_price_tag,
            OHBE_Tools::formatPrice(
                floatval($total_rate_orig_price) / $acco['nights']
            )
        );
        if ($default_rate_has_promo) {
            $price = floatval(isset($selected_acco)
                ? $selected_acco['price'] : $first_price_key);
            $html .= sprintf(
                '<div class=promo>'
                . '<div class="ohbe-title tag">%s</div>'
                . '<div class=value>%s</div></div>',
                $acco['promo_code'],
                OHBE_Tools::formatPrice($price / $acco['nights'])
            );
        }
        // Rates list
        $full_rate_structure = '';
        $rate_list_final = '';
        foreach ($acco['periods'] as $rate_price => $sorted_period) {
            foreach ($sorted_period as $sorted => $period_parts) {
                $rate_has_promo = false;
                $total_rate_orig_price = 0;
                $data_orig_price = '';
                if (count($period_parts) == 1) {
                    // UNCOMBINED RATE
                    $rate_list = OHBE_Availability::renderUncombinedRateList(
                        $acco['nights'],
                        $period_parts[0],
                        $first_price_key,
                        $rates,
                        $rate_price,
                        $sorted,
                        $best_price_tag,
                        $total_rate_orig_price,
                        $full_rate_structure,
                        $rate_has_promo
                    );
                }
                else {
                    // COMBINED RATES
                    $rate_list = OHBE_Availability::renderCombinedRateList(
                        $acco['nights'],
                        $period_parts,
                        $first_price_key,
                        $rates,
                        $rate_price,
                        $sorted,
                        $best_price_tag,
                        $total_rate_orig_price,
                        $full_rate_structure,
                        $rate_has_promo
                    );
                }
                if ($rate_has_promo) {
                    $data_orig_price = sprintf(
                        ' data-orig-price=%s',
                        $total_rate_orig_price
                    );
                    $rate_list .= sprintf(
                        '<div class=total-promo>'
                        . '<div class="ohbe-title tag">%s</div>'
                        . '<div class=value>%s</div>'
                        . '</div>',
                        $acco['promo_code'],
                        OHBE_Tools::formatPrice($rate_price / $acco['nights'])
                    );
                }
                if ($selected_acco) {
                    $show_selected = $selected_acco['rate'] == $full_rate_structure
                        ? ' selected' : '';
                }
                else {
                    $show_selected = $rate_price == $first_price_key && $sorted == 0
                        ? ' selected' : '';
                }
                $rate_list_final .= sprintf(
                    '<li class="rate%s" data-price="%s"%s '
                    . 'data-sorted=%s data-adults=%s data-children=%s>%s',
                    $show_selected,
                    $rate_price,
                    $data_orig_price,
                    $sorted,
                    $period_parts[0]['adults'],
                    $period_parts[0]['children'],
                    $rate_list
                );
            }
            $rate_list_final .= '</li>';
        }
        $html .= sprintf(
            '</div><button class=btn-select-rate>%s'
            . '<span class="dashicons dashicons-arrow-down"></span></button>'
            . '<ul class=rate-list>%s</ul>%s</div></div>',
            __('Change rate', 'ohbe'),
            $rate_list_final,
            OHBE_Availability::renderAccosTotalBlock($acco, $default_rate_has_promo)
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate info block.
     *
     * @param  array $acco Single accommodation structure.
     * @param  array $selected_acco Single selected accommodation structure.
     * @param array $rates Rates info from Oscar availability call.
     * @param string $first_price_key First key of periods structure.
     * @param string $total_rate_orig_price Total rate original price.
     * @param bool $default_rate_has_promo Tell if the default rate has promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderUncombinedRateInfo(
        $acco,
        $selected_acco,
        $rates,
        $first_price_key,
        &$total_rate_orig_price,
        &$default_rate_has_promo
    ) {
        if ($selected_acco) {
            // This accommodation has been selected previously
            $rate_id = $selected_acco['rate'];
            $sorted = $selected_acco['sorted'];
            if (isset($acco['orig_price'])) {
                $default_rate_has_promo = true;
                $data_prices = sprintf(
                    'data-price=%s data-orig-price=%s',
                    $selected_acco['price'],
                    $acco['orig_price']
                );
            }
            else {
                $data_prices = sprintf(
                    'data-price=%s',
                    $selected_acco['price']
                );
            }
        }
        else {
            // This accommodation hasn't been selected previously
            $rate_id = $acco['periods'][$first_price_key][0][0]['rate_id'];
            $sorted = 0;
            if (isset($acco['orig_price'])) {
                $default_rate_has_promo = true;
                $data_prices = sprintf(
                    'data-price=%s data-orig-price=%s',
                    $acco['price'],
                    $acco['orig_price']
                );
            }
            else {
                $data_prices = sprintf(
                    'data-price=%s',
                    $acco['price']
                );
            }
        }
        $total_rate_orig_price =
            isset($acco['orig_price'])
                ? $acco['orig_price']
                : $acco['price'];
        $html = sprintf(
            '<div class=info data-amount=%s data-sorted=%s %s>',
            $acco['amount'],
            $sorted,
            $data_prices
        );
        $rate_name = sprintf(
            '%s %s',
            OHBE_Tools::renderOccupancyIcons($rate_id, 'rates'),
            htmlspecialchars($rates[$rate_id]['name'])
        );
        $html .= sprintf(
            '<div class=name>%s</div>'
            . '<div class="conditions modal-button" '
            . 'role=button data-modal=conditions-modal>'
            . '<a><span class="dashicons dashicons-info"></span> %s</a></div>',
            $rate_name,
            __('Conditions', 'ohbe')
        );

        return $html;
    }

    /**
     * Render accommodation card's combined rate list block.
     *
     * @param  int $nights Nights.
     * @param  array $rate Rate single period part.
     * @param string $first_price_key First key of periods structure.
     * @param array $rates Rates info from Oscar availability call.
     * @param string $rate_price Rate price.
     * @param string $sorted Order of the rate (used when there are more than one
     * with the same price).
     * @param string $best_price_tag Best price tag's html structure
     * @param string $total_rate_orig_price Total rate original price.
     * @param string $full_rate_structure Empty string to be filled and get all
     * rate ids in a row
     * @param bool $rate_has_promo Tell if the rate has promo or not.
     * @return string Accos-total block's html structure.
     *
     */
    public static function renderUncombinedRateList(
        $nights,
        $rate,
        $first_price_key,
        $rates,
        $rate_price,
        $sorted,
        $best_price_tag,
        &$total_rate_orig_price,
        &$full_rate_structure,
        &$rate_has_promo
    ) {
        $full_rate_structure = $rate['rate_id'];
        $rate_has_promo = isset($rate['orig_price']);
        $rate_name = sprintf(
            '%s %s',
            OHBE_Tools::renderOccupancyIcons($rate['rate_id'], 'rates'),
            htmlspecialchars($rates[$rate['rate_id']]['name'])
        );
        $total_rate_orig_price = isset($rate['orig_price'])
            ? $rate['orig_price'] : $rate['price'];

        return sprintf(
            '<div class=ohbe-title>%s</div>'
            . '<div class="value%s">%s %s</div>',
            $rate_name,
            isset($rate['orig_price']) ? ' orig' : '',
            !$rate_has_promo
            && $first_price_key == $rate_price
            && $sorted == 0 ? $best_price_tag : '',
            OHBE_Tools::formatPrice(floatval($total_rate_orig_price) / $nights)
        );
    }
}
