<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Reservation functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Reservation
{
    /**
     * Get translated name label.
     *
     * @param string $field_id Name of the field
     * @return string Name of label.
     */
    public static function getFieldLabel($field_id) {
        switch ($field_id) {
            case 'firstnames':
                return __('First names', 'ohbe');
            case 'middlenames':
                return __('Middle names', 'ohbe');
            case 'surname':
                return __('Surname', 'ohbe');
            case 'surname2':
                return __('Surname 2', 'ohbe');
            case 'address':
                return __('Address', 'ohbe');
            case 'town':
                return __('Town', 'ohbe');
            case 'state':
                return __('State', 'ohbe');
            case 'postalcode':
                return __('Postal code', 'ohbe');
            case 'phone':
                return __('Phone', 'ohbe');
            case 'phone2':
                return __('Phone 2', 'ohbe');
            case 'email':
                return __('Email', 'ohbe');
            case 'country':
                return __('Country', 'ohbe');
            case 'arrival_time':
                return __('Arrival time', 'ohbe');
            case 'departure_time':
                return __('Departure time', 'ohbe');
            case 'card_issuer':
                return __('Issuer', 'ohbe');
            case 'card_number':
                return __('Number', 'ohbe');
            case 'card_csc':
                return __('CSC', 'ohbe');
            case 'card_holder':
                return __('Holder', 'ohbe');
            case 'card_expires':
                return __('Expiry', 'ohbe');
        }
    }

    /**
     * Set param for item list call.
     *
     * @param string $key Param's key.
     * @param mixed $value Param's value.
     * @return void
     */
    public static function setReservationParam($key, $value) {
        if (!isset($_SESSION['ohbe']['reservation'])) {
            $_SESSION['ohbe']['reservation'] = array();
        }
        $_SESSION['ohbe']['reservation'][$key] = $value;
    }

    /**
     * Set accommodation structure for Oscar reservation call.
     *
     * @return array Accommodations list ready for API call.
     */
    public static function getAccommodationStructure() {
        $accos = array();
        foreach ($_SESSION['ohbe']['selected_accos'] as $sa) {
            for ($i=0; $i<$sa['quantity']; $i++) {
                $acco = array(
                    'adults' => intval($sa['adults']),
                    'children' => intval($sa['children']),
                    'acco_id' => intval($sa['acco_id']),
                    'price' => $sa['price'],
                    'periods' => [],
                );
                foreach ($sa['periods'] as $p) {
                    $acco['periods'][] = array(
                        'rate_id' => intval($p['rate_id']),
                        'days' => intval($p['days']),
                    );
                }
                $accos[] = $acco;
            }
        }

        return $accos;
    }

    /**
     * Get the values of every field when a reservation has been made.
     *
     * @return array|string List of reservation params or string error.
     */
    public static function getReservationFieldValues() {
        $reservation_data = array();
        foreach ($_POST as $key => $value) {
            // Add only values that contains 'ohbe_'
            // at the beginning of the string
            if (strpos($key, 'ohbe_') === 0) {
                // Accommodations sanitization
                if (is_array($value)) {
                    for ($i=0; $i<count($value); $i++) {
                        $reservation_data[$key][$i] = $value[$i];
                    }
                }
                else {
                    // Fields sanitization
                    $reservation_data[$key] = $value;
                }
            }
        }
        if (
            array_key_exists('ohbe_card_expires_month', $reservation_data)
            && array_key_exists('ohbe_card_expires_year', $reservation_data)
        ) {
            $card_expires_date = DateTime::createFromFormat(
                'Y-m-d',
                $reservation_data['ohbe_card_expires_year']
                . '-'
                . $reservation_data['ohbe_card_expires_month']
                . '-01'
            );
            if (!$card_expires_date) {
                return 'wrongDate';
            }
            $card_expires_date->modify('+1 month');
            $card_expires_date->modify('-1 day');
            $reservation_data['ohbe_card_expires'] = $card_expires_date->format(
                'Y-m-d'
            );
        }

        return $reservation_data;
    }

    /**
     * Get the param/s for item list call.
     *
     * @param mixed $param A single reservation parameter
     * @return array
     */
    public static function getReservationParam($param = null) {
        if (isset($_SESSION['ohbe']['reservation'])) {
            $reservation = $_SESSION['ohbe']['reservation'];
            return isset($param) ? $reservation[$param] : $reservation;
        }

        return null;
    }

    /**
     * Print credit card header when some credit card field
     * is set in inventory.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return void
     */
    public static function printCreditCardDataHeader($redforts_data) {
        $updated_oscar_data = OHBE_Tools::checkVersion(
            $redforts_data['inventory_version']
        );

        if ($updated_oscar_data) {
            OscarAPI::getInventoryData();
        }
        $fields = OHBE_Main::getInventory('fields');

        foreach ($fields as $field) {
            if (strncmp($field['id'], "card", 4) == 0) {
                printf(
                    '<h3 class="ohbe-title">%s</h3>',
                    __('Credit card data', 'ohbe')
                );
                return;
            }
        }
    }

    /**
     * Print the rate(s) occupancy icons
     *
     * @param bool $isRemovable Tell when the accommodation card is removable.
     * @return void
     */
    public static function printRemovable($isRemovable) {
        if ($isRemovable) {
            print(
                '<div class="delete"><a class="delete-acco" role="button">'
                . '<span class="dashicons dashicons-no"></span></a></div>'
            );
        }
    }

    /**
     * Print a block of fields.
     *
     * @param string $data_type Block type.
     * @param array $redforts_data Oscar availability call response.
     * @param object $errors When errors exist.
     * @return void
     */
    public static function printReservationFields(
        $data_type,
        $redforts_data,
        $errors
    ) {
        $updated_oscar_data = OHBE_Tools::checkVersion(
            $redforts_data['inventory_version']
        );
        if ($updated_oscar_data) {
            OscarAPI::getInventoryData();
        }
        $fields = OHBE_Main::getInventory('fields');
        foreach ($fields as $field) {
            self::printReservationFieldType($field, $data_type, $errors);
        }
    }

    /**
     * Print field.
     *
     * @param array $field Inventory field element.
     * @param string $data_type Block type.
     * @param object $errors When errors exist.
     * @return void
     */
    public static function printReservationFieldType($field, $data_type, $errors) {
        $label = self::getFieldLabel($field['id']);

        switch($data_type) {
            case 'personal':
                switch ($field['id']) {
                    case 'firstnames':
                    case 'middlenames':
                    case 'surname':
                    case 'surname2':
                    case 'address':
                    case 'town':
                    case 'state':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput($field)
                        );
                        break;
                    case 'postalcode':
                    case 'phone':
                    case 'phone2':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel'
                            )
                        );
                        break;
                    case 'email':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'email'
                            )
                        );
                        break;
                    case 'country':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                $errors
                            )
                        );
                        break;
                    case 'arrival_time':
                    case 'departure_time':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s %s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            _e('Check in/out conditions', 'ohbe'),
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                $errors
                            ),
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                $errors
                            )
                        );
                        break;
                }
                break;

            case 'card':
                switch ($field['id']) {
                    case 'card_issuer':
                        if (
                            OHBE_Main::getInventory('issuers')
                            && count(OHBE_Main::getInventory('issuers')) > 0
                        ) {
                            printf(
                                '<div class=reservation-field>'
                                . '<label for=%s>%s%s</label>%s</div>',
                                'ohbe_' . htmlspecialchars($field['id']),
                                $label,
                                $field['required']
                                    ? '<span class=ohbe-field-required>*</span>' : '',
                                OHBE_Reservation::renderReservationFieldDropdown(
                                    $field,
                                    $errors
                                )
                            );
                        }
                        break;
                    case 'card_number':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel',
                                'card_number'
                            )
                        );
                        break;
                    case 'card_csc':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput(
                                $field,
                                'tel',
                                'card_csc'
                            )
                        );
                        break;
                    case 'card_holder':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s</div>',
                            'ohbe_' . htmlspecialchars($field['id']),
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldInput($field)
                        );
                        break;
                    case 'card_expires':
                        printf(
                            '<div class=reservation-field>'
                            . '<label for=%s>%s%s</label>%s %s</div>',
                            'ohbe_' . htmlspecialchars($field['id']) . '_month',
                            $label,
                            $field['required']
                                ? '<span class=ohbe-field-required>*</span>' : '',
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                $errors,
                                '_month'
                            ),
                            OHBE_Reservation::renderReservationFieldDropdown(
                                $field,
                                $errors,
                                '_year'
                            )
                        );
                        break;
                }
                break;
        }
    }

    /**
     * Render accommodation card's name.
     *
     * @param array $acco Single accommodation structure.
     * @return string
     */
    public static function renderAccommodationCardName($acco) {
        $quantity = '';
        if ($acco['quantity'] > 1) {
            $quantity = sprintf('%s × ', $acco['quantity']);
        }

        return $quantity . $acco['name'];
    }

    /**
     * Render conditions checkbox.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return string Checkbox's html structure.
     */
    public static function renderConditions($redforts_data) {
        $html = '';

        if (
            OHBE_Main::getSettings('conditions')
            && (isset($redforts_data['checkinout'])
                || isset($redforts_data['conditions']))
        ) {
            $link = sprintf(__(
                    'I have read the <a %s>reservation conditions</a>'
                    . ' and I accept it.'
                    , 'ohbe'
                ),
                'class="modal-button" role="button" data-modal=conditions-modal'
            );
            $html = '<div class=ohbe-conditions>'
                . '<h3 class=ohbe-title>' . __('Conditions', 'ohbe') . '</h3>'
                . '<input type=checkbox name=accept_conditions required> '
                . $link
                . '<span class=ohbe-field-required>*</span>'
                . '</div>';
        }

        return $html;
    }

    /**
     * Render consent block.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return string Consent block html structure.
     */
    public static function renderConsent($redforts_data) {
        $html = '';

        if (
            isset($redforts_data['consent_title'])
            && isset($redforts_data['consent_text'])
        ) {
            $html .= sprintf(
                '<h3 class=ohbe-title>%s</h3>'
                . '<label for=ohbe_consent class=ohbe-checkbox-label>'
                . '<input id=ohbe_consent type=checkbox name=ohbe_consent> %s'
                . '</label>',
                htmlspecialchars($redforts_data['consent_title']),
                nl2br(htmlspecialchars($redforts_data['consent_text']))
            );
        }

        return $html;
    }

    /**
     * Render modal.
     *
     * @param string $id Name of the modal.
     * @param string $title Translated title.
     * @param string $content Modal's content.
     * @return string Modal's html structure.
     */
    public static function renderModal($id, $title, $content) {
        return sprintf(
            '<div id=%1$s-modal class=ohbe-modal>'
            . '<div class=modal-dialog role=document>'
            . '<div class=modal-content>'
            . '<div class=modal-header>'
            . '<span class=close>&times;</span>'
            . '<h4 class=modal-title>%2$s</h4>'
            . '</div>'
            . '<div class=modal-body>%3$s</div><div class=modal-footer>'
            . '<button type=button class="btn-ohbe btn-modal accept">%4$s</button>'
            . '</div></div></div></div>',
            $id,
            $title,
            $content,
            __('Close', 'ohbe')
        );
    }

    /**
     * Render the accommodations selected in the availability page.
     * Display every accommodation cards.
     *
     * @param int $total_price Accommodations' total price.
     * @param array $rates (used in accommodation_reservation_card.php).
     * @return void
     */
    public static function printOscarAccommodationsReservation(
        &$total_price,
        $rates
    ) {
        $selected_accommodations = array();

        // Reservation
        if (
            $_SERVER['REQUEST_METHOD'] == "POST"
            && isset($_POST['ohbe_go_reservation_page'])
            && isset($_POST['accommodations'])
        ) {
            $accommodations_exchangeable = json_decode(
                html_entity_decode(stripslashes($_POST['accommodations'])),
                true
            );
        }
        else if (
            isset($_SESSION['ohbe']['selected_accos'])
            && !empty($_SESSION['ohbe']['selected_accos'])
        ) {
            $accommodations_exchangeable = $_SESSION['ohbe']['selected_accos'];
        }

        $_SESSION['ohbe']['selected_accos'] = array();
        $isRemovable = false;

        if (isset($accommodations_exchangeable)) {
            foreach ($accommodations_exchangeable as $acco) {
                $_SESSION['ohbe']['selected_accos'][] = $acco;
                $selected_accommodations[] = $acco;
                $total_price += $acco['quantity'] * $acco['price'];
                include(
                    OHBE_PATH . 'includes/templates/views/'
                    . 'accommodation_reservation_card.php'
                );
            }
        }
    }

    /**
     * Render privacy statement block.
     *
     * @param array $redforts_data Oscar availability call response.
     * @return string Privacy statement block html structure.
     */
    public static function renderPrivacyStatement($redforts_data) {
        $html = '';

        if (
            isset($redforts_data['privacy'])
            || isset($redforts_data['privacy_summary'])
        ) {
            $html .= '<h3 class=ohbe-title>'
                . __('Privacy statement', 'ohbe')
                . '</h3>';
        }

        if (isset($redforts_data['privacy_summary'])) {
            $html .= nl2br(htmlspecialchars($redforts_data['privacy_summary']))
                . ' ';
        }

        if (isset($redforts_data['privacy'])) {
            $html .= '<span class=ohbe-privacy>'
                . sprintf(
                    __(
                        'For more information please consult the '
                        . '<a %s>privacy statement</a>'
                        , 'ohbe'
                    ),
                    'class=modal-button role=button data-modal=privacy-modal'
                )
                . '</span>';
        }

        return $html;
    }

    /**
     * Render the name(s) of rate(s)
     *
     * @param array $periods Accommodation periods structure.
     * @param array $rates (used in accommodation_reservation_card.php).
     * @return string Name(s) of rate(s).
     */
    public static function renderRateStrippedNames($periods, $rates) {
        $rate_names = array();
        foreach ($periods as $period) {
            $rate_names[] = $rates[$period['rate_id']]['name'];
        }

        return htmlspecialchars(join(', ', $rate_names));
    }

    /**
     * Render the rate(s) occupancy icons
     *
     * @param array $acco Accommodation.
     * @param array $rates (used in accommodation_reservation_card.php).
     * @return string Name(s) of rate(s).
     */
    public static function renderRateOccupancyIcons($acco, $rates) {
        $html = '';
        for ($i=0; $i<count($acco['periods']); $i++) {
            foreach ($rates as $key => $rate) {
                if ($key == $acco['periods'][$i]['rate_id']) {
                    $html .= OHBE_Tools::renderOccupancyIcons($key, 'rates');
                    break;
                }
            }
        }
        if ($html == '') {
            $html .= OHBE_Tools::renderOccupancyIcons($acco['acco_id'], 'accos');
        }

        return $html;
    }

    /**
     * Render dropdown field.
     *
     * @param array $field Inventory field element.
     * @param object $errors When errors exist.
     * @param string $extra_data Used for differentiation (_month|_year).
     * @return string Dropdown's html structure.
     */
    private static function renderReservationFieldDropdown(
        $field,
        $errors,
        $extra_data=''
    ) {
        $error_border = isset($errors) && $errors->get_error_message(
            'ohbe_' . htmlspecialchars($field['id'])
        )
            ? ' error-border'
            : '';
        $sanizite_error = isset($errors) && count($errors->errors) > 0
            ? 'value="' . htmlspecialchars(
                $_POST['ohbe_' . $field['id'] . $extra_data]
            ) . '"'
            : '';
        $required = $field['required'] ? ' required aria-required="true"' : '';
        $middle = $extra_data == '_month' || $extra_data == '_year' ? ' middle' : '';
        $extra_data_value = $extra_data != '' ? $extra_data : '';
        $html = sprintf(
            '<select id=%1$s name="%1$s%6$s"%2$s '
            . 'class="ohbe-select ohbe-field%3$s%5$s" %4$s>',
            'ohbe_' . $field['id'],
            $required,
            $error_border,
            $sanizite_error,
            $middle,
            $extra_data_value
        );

        // Extra data gets the values '_month' or '_year',
        // those are used for 'card_expires' values.
        switch($field['id'] . $extra_data) {
            case 'country':
                $html .= '<option selected disabled></option>';
                $html .= OHBE_Tools::getSelectCountries();
                break;
            case 'arrival_time':
                $html .= '<option selected disabled>'
                    . __('Arrival', 'ohbe')
                    . '</option>'
                    . file_get_contents(OHBE_PATH . 'includes/fields/time.html');
                break;
            case 'departure_time':
                $html .= '<option selected disabled>'
                    . __('Departure', 'ohbe')
                    . '</option>'
                    . file_get_contents(OHBE_PATH . 'includes/fields/time.html');
                break;
            case 'card_issuer';
                foreach (OHBE_Main::getInventory('issuers') as $issuer) {
                    $html .= sprintf(
                        '<option value="%1$s">%1$s</option>',
                        htmlspecialchars($issuer)
                    );
                }
                break;
            case 'card_expires_month':
                $html .= '<option selected>'
                    . __('Month', 'ohbe')
                    . '</option>'
                    . file_get_contents(OHBE_PATH . 'includes/fields/months.html');
                break;
            case 'card_expires_year':
                $html .= '<option selected>'
                    . __('Year', 'ohbe')
                    . '</option>';
                for ($i=0; $i<=10; $i++) {
                    $html .= sprintf(
                        '<option value="%1$s">%1$s</option>',
                        date("Y") + $i
                    );
                }
                break;
        }

        $html .= '</select>';

        return $html;
    }

    /**
     * Render input field.
     *
     * @param array $field Inventory field element.
     * @param string $type Type of input.
     * @param string $extra_data Used for differentiation (card_number|card_csc).
     * @return string Input's html structure.
     */
    private static function renderReservationFieldInput(
        $field,
        $type='text',
        $extra_data = ''
    ) {
        $error_border = isset($errors) && $errors->get_error_message(
                'ohbe_' . htmlspecialchars($field['id'])
            )
            ? 'error-border'
            : '';
        $sanizite_error = isset($errors) && count($errors->errors) > 0
            ? 'value="' . htmlspecialchars($_POST['ohbe_' . $field['id']]) . '"'
            : '';
        $required = $field['required'] ? 'required aria-required="true"' : '';
        $extra_data_value = '';
        switch($extra_data) {
            case 'card_number':
                $extra_data_value = 'data-regexp="([0-9][^0-9]*){15,}\"';
                break;
            case 'card_csc':
                $extra_data_value = 'data-regexp="([0-9]{3,4})\"';
                break;
        }

        return sprintf(
            '<input id=%1$s name=%1$s %2$s '
            . 'class="ohbe-select ohbe-field %3$s" %3$s type="%5$s" %4$s %6$s>',
            'ohbe_' . $field['id'],
            $required,
            $error_border,
            $sanizite_error,
            $type,
            $extra_data_value
        );
    }

    /**
     * Render taxes information.
     *
     * @param int $excl_taxes Quantity of taxes exclusion (%).
     * @return string Translated taxes information.
     */
    public static function renderTaxesString($excl_taxes) {
        return $excl_taxes > 0
            ? sprintf(__('Price excl. %s%% taxes', 'ohbe'), $excl_taxes)
            : __('Price incl. taxes', 'ohbe');
    }
}
