<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Functions for different tasks
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */
class OHBE_Tools
{
    /**
     * Check whether it is needed to update the inventory.
     *
     * @param int $version Number given by the Oscar inventory call
     * to know if inventory updated is needed.
     * @return bool
     */
    public static function checkVersion($version) {
        if (isset($version) && !OHBE_Main::getInventory('version')) {
            return true;
        }
        if (
            isset($version) && OHBE_Main::getInventory('version')
            && ($version > (int) OHBE_Main::getInventory('version'))
        ) {
            return true;
        }

        return false;
    }

    /**
     * @param string $filters Url filters. Divided  by "!"
     * @return mixed
     * array: All filters.
     * int: No filters.
     * object: Error(s).
     */
    public static function decodeFilters($filters) {
        if ($filters != '0') {
            $all_filters = array();
            $filters = explode('!', substr($filters, 0, -1));

            foreach ($filters as $f) {
                $ft = explode(':', $f);
                $all_filters[$ft[0]] = $ft[1];
            }
            // Arrival date (Check in)
            if (isset($all_filters['ci'])) {
                $arrival_date = new DateTime();
                if (is_numeric($all_filters['ci'])) {
                    $arrival_date->setTimestamp($all_filters['ci']);
                }
                $all_filters['ci'] = $arrival_date;
            }
            else {
                $errors = OHBE_Availability::accommodationAvailabilityErrors(
                    'invalid_param'
                );
                return $errors;
            }
            // Departure date (Check out)
            if (isset($all_filters['co'])) {
                $departure_date = new DateTime();
                if (is_numeric($all_filters['co'])) {
                    $departure_date->setTimestamp($all_filters['co']);
                }
                else {
                    $departure_date->modify('+1 day');
                }
                $all_filters['co'] = $departure_date;
            }
            else {
                $errors = OHBE_Availability::accommodationAvailabilityErrors(
                    'invalid_param'
                );
                return $errors;
            }

            return $all_filters;
        }
        else {
            return 0;
        }
    }

    /**
     * Encode search box's filters.
     *
     * @return int|string Filters encoded.
     */
    public static function encodeFilters() {
        $encoded_filters = '';
        // Arrival date (Check in)
        if (isset($_POST['ohbe_check_in'])) {
            $arrival_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_check_in']
            );
            $encoded_filters .= sprintf(
                'ci:%s!',
                date_timestamp_get($arrival_date)
            );
        }
        // Departure date (Check out)
        if (isset($_POST['ohbe_check_out'])) {
            $departure_date = DateTime::createFromFormat(
                '!d-m-Y',
                $_POST['ohbe_check_out']
            );
            $encoded_filters .= sprintf(
                'co:%s!',
                date_timestamp_get($departure_date)
            );
        }
        // Promo code
        if (isset($_POST['ohbe_promo']) && !empty($_POST['ohbe_promo'])) {
            $encoded_filters .= sprintf(
                'pr:%s!',
                htmlspecialchars($_POST['ohbe_promo'])
            );
        }
        if ($encoded_filters == '') {
            $encoded_filters = 0;
        }

        return $encoded_filters;
    }

    /**
     * Get formatted price depending on inventory currency data.
     *
     * @return string Formatted price.
     */
    public static function formatPrice($price) {
        $symbol = OHBE_Main::getInventory('currency', 'symbol');
        $formatted_price = number_format(
            $price,
            OHBE_Main::getInventory('currency', 'digits')
        );

        return OHBE_Main::getInventory('currency', 'before')
            ? sprintf('%s %s', $symbol, $formatted_price)
            : sprintf('%s %s', $formatted_price, $symbol);
    }

    /**
     * Get home url with language slug.
     *
     * @return string Url
     */
    public static function getBaseUrl() {
        // Get home url with polylang
        if (
            function_exists('pll_home_url')
            && pll_home_url(pll_current_language('slug'))
        ) {
            $base_url = pll_home_url(pll_current_language('slug'));

            if (substr($base_url, -1) !== '/') {
                $base_url .= '/';
            }
        }
        else {
            $base_url = home_url('/');
        }

        return $base_url;
    }

    /**
     * Get nights interval between arrival and departure.
     *
     * @param array $acco Accommodation.
     * @return string Nights' quantity.
     */
    public static function getNightsInterval($acco) {
        $datetime1 = new DateTime($acco['arrival']);
        $datetime2 = new DateTime($acco['departure']);
        $interval = $datetime1->diff($datetime2);

        return $interval->format("%d");
    }

    /**
     * Get filters from search form.
     *
     * @param string $formatted Format for dates. Example: 'Y-m-d'.
     * @return array Filters.
     */
    public static function getSearchFilters($formatted=null) {
        if (isset($_GET['ohbe'])) {
            $all_filters = self::decodeFilters($_GET['ohbe']);
        }
        else {
            $all_filters['ci'] = '';
            $all_filters['co'] = '';
        }
        $search_filters = array(
            'check_in' => $all_filters['ci'],
            'check_out' => $all_filters['co'],
            'promotion_code' => isset($_GET['ohbe']) && isset($all_filters['pr'])
                ? $all_filters['pr'] : ''
        );
        if ($formatted && isset($_GET['ohbe'])) {
            $search_filters['check_in'] = $all_filters['ci']->format($formatted);
            $search_filters['check_out'] = $all_filters['co']->format($formatted);
        }

        return $search_filters;
    }

    /**
     * Get the list of countries as select's options
     *
     * @return string Country options
     */
    public static function getSelectCountries() {
        $html = '';
        $countries = include OHBE_PATH . 'includes/fields/countries.php';

        foreach ($countries as $key => $value) {
            $html .= '<option value=' . $key . '>'
                . __($value, 'ohbe-countries')
                . '</option>';
        }

        return $html;
    }

    /**
     * Get accommodation from selected accommodation list.
     *
     * @param array $acco Accommodation to compare with.
     * @return array Selected accommodation.
     */
    public static function getSelectedAcco($acco) {
        if (isset($_SESSION['ohbe']['selected_accos'])) {
            foreach ($_SESSION['ohbe']['selected_accos'] as $selected_acco) {
                if ($selected_acco['acco_id'] == $acco['acco_id']) {
                    return $selected_acco;
                }
            }
        }

        return null;
    }

    /**
     * Get localized parameters for search datepicker.
     *
     * @return array Localized parameters.
     */
    public static function localizeDatepicker() {
        global $wp_locale;
        $datepicker_args = array(
            'monthNames' => self::stripArrayIndexes($wp_locale->month),
            'monthNamesShort' => self::stripArrayIndexes($wp_locale->month_abbrev),
            'dayNames' => self::stripArrayIndexes($wp_locale->weekday),
            'dayNamesShort' => self::stripArrayIndexes($wp_locale->weekday_abbrev),
            'dayNamesMin' => self::stripArrayIndexes($wp_locale->weekday_initial),
            'firstDay' => get_option('start_of_week'),
            'isRTL' => ($wp_locale->text_direction == 'rtl') ? true : false,
        );

        return $datepicker_args;
    }

    /**
     * Print flash message.
     *
     * @return void
     */
    public static function printFlashMessage() {
        if (isset($_SESSION['ohbe']['flash_message'])) {
            printf(
                '<div class="alert alert-%s">%s</div>',
                $_SESSION['ohbe']['flash_message']['type'],
                $_SESSION['ohbe']['flash_message']['message']
            );
            unset($_SESSION['ohbe']['flash_message']);
        }
    }

    /**
     * Render accommodation image.
     *
     * @param int $acco_id Accommodation id.
     * @return string Accommodation image's html structure.
     */
    public static function renderAccoImage($acco_id) {
        $accos = OHBE_Main::getInventory('accos');

        if ($accos) {
            foreach ($accos as $acco) {
                if ($acco['id'] == $acco_id && isset($acco['image_url'])) {
                    return sprintf(
                        '<div class=acco-image>'
                        . '<img src=%s></div>',
                        $acco['image_url']
                    );
                }
            }
        }

        return '';
    }

    /**
     * Render occupancy icons for a rate.
     *
     * @param string $id Rate id
     * @param string $type Type of list to get icons from.
     * @return string Occupancy icon's html structure.
     */
    public static function renderOccupancyIcons($id, $type) {
        $list = OHBE_Main::getInventory($type);
        $occupancy = '';

        if ($list) {
            foreach ($list as $item) {
                if (
                    isset($item['adults']) && isset($item['adults'])
                    && $item['id'] == $id
                ) {
                    if ($item['adults'] > 5) {
                        $occupancy .= sprintf(
                            '<div class="dashicons dashicons-admin-users" '
                            . 'title="%s"></div>×%s',
                            __('Adults', 'ohbe'),
                            $item['adults']
                        );
                    }
                    else {
                        for ($i=0; $i<$item['adults']; $i++) {
                            $occupancy .= sprintf(
                                '<div class="dashicons dashicons-admin-users" '
                                . 'title="%s"></div>',
                                __('Adults', 'ohbe')
                            );
                        }
                    }
                    if (
                        $item['adults']
                        && $item['children']
                        && ($item['adults'] > 5 || $item['children'] > 5)
                    ) {
                        $occupancy .= ' + ';
                    }
                    if ($item['children'] > 5) {
                        $occupancy .= sprintf(
                            '<div class="dashicons dashicons-universal-access" '
                            . 'title="%s"></div>×%s',
                            __('Children', 'ohbe'),
                            $item['children']
                        );
                    }
                    else {
                        for ($i=0; $i<$item['children']; $i++) {
                            $occupancy .= sprintf(
                                '<div class="dashicons dashicons-universal-access" '
                                . 'title="%s"></div>',
                                __('Children', 'ohbe')
                            );
                        }
                    }
                }

            }
        }

        return $occupancy;
    }

    /**
     * Set flash message.
     *
     * @param string $message Message to be displayed.
     * @param string $type Type of alert [success, danger].
     * @return void
     */
    public static function setFlashMessage($message, $type = 'success') {
        $_SESSION['ohbe']['flash_message'] = array(
            'message' => $message,
            'type' => $type
        );
    }

    /**
     * Sort price by ASC.
     *
     * @return int Difference between prices.
     */
    public static function sortByPrice($a, $b) {
        return $a['price'] - $b['price'];
    }

    /**
     * Remove "string" indexes from array.
     *
     * @return array Array without "string" indexes
     */
    public static function stripArrayIndexes($array) {
        foreach ($array as $item) {
            $array_stripped[] = $item;
        }

        return $array_stripped;
    }
}
