<?php
defined('ABSPATH') or exit();

/**
 * Redforts Oscar Hotel Booking
 * Front core functions
 *
 * @author Redforts Software S.L.
 * @copyright Copyright (c) 2015 Redforts Software S.L.
 * @link https://redforts.com
 * @since 0.1.1
 */

/**
 * Enqueue Dashicons style to frontend.
 *
 * @return void
 */
function addDashicons() {
    wp_enqueue_style('dashicons');
}

add_action('wp_enqueue_scripts', 'addDashicons');

/**
 * Handles accommodations availability page.
 *
 * @return mixed
 *      array When param "ohbe" is set & filters are set
 *      int When param "ohbe" is set & no filters are set
 *      null When param "ohbe" is not set & no errors are set
 *      object When errors are set
 */
function ohbeAvailabilityFormHandle() {
    if (isset($_GET['ohbe'])) {
        $all_filters = OHBE_Tools::decodeFilters($_GET['ohbe']);
        if (array_key_exists('errors', $all_filters)) {
            return $all_filters;
        }
        $errors = OHBE_Availability::accommodationAvailabilityErrors(
            'position',
            $all_filters
        );
    }
    else {
        $errors = OHBE_Availability::accommodationAvailabilityErrors('params');
    }
    if ($errors) {
        return $errors;
    }
}

/**
 * Handle search errors.
 *
 * @return object Search errors.
 */
function ohbeSearchErrors() {
    $errors_ohbe = new WP_Error();

    if (
        !isset($_POST['ohbe_check_in'])
        || empty($_POST['ohbe_check_in'])
        || !preg_match('/[0-9]{2}-[0-9]{2}-[0-9]{4}/', $_POST['ohbe_check_in'])
    ) {
        $errors_ohbe->add('ohbeCheckIn', __('Invalid arrival date', 'ohbe'));
        return $errors_ohbe;
    }

    if (
        !isset($_POST['ohbe_check_out'])
        || empty($_POST['ohbe_check_out'])
        || !preg_match('/[0-9]{2}-[0-9]{2}-[0-9]{4}/', $_POST['ohbe_check_out'])
    ) {
        $errors_ohbe->add('ohbeCheckOut', __('Invalid departure date', 'ohbe'));
        return $errors_ohbe;
    }
}

/**
 * Handle accommodations search form.
 * Redirect to availability page.
 *
 * @return object When errors are set
 */
function ohbeSearchFormHandle($id, $ispr) {
    $errors = ohbeSearchErrors();
    if ($errors) {
        return $errors;
    }
    $encoded_filters = OHBE_Tools::encodeFilters();
    $params = array('ohbe' => $encoded_filters);
    if ($id) {
        $params['acco'] = $id;
    }
    if ($ispr == 1) {
        $params['ispr'] = '';
    }
    // Remove selected accommodations previously
    if (isset($_SESSION['ohbe']['selected_accos'])) {
        unset($_SESSION['ohbe']['selected_accos']);
    }
    $url = esc_url_raw(add_query_arg($params,  '/?' . OHBE_AVAILABILITY_PAGE));
    wp_safe_redirect(OHBE_Tools::getBaseUrl() . $url);
    exit;
}

/**
 * Handle reservation errors.
 *
 * @return object|void
 */
function ohbeReservationErrors($api_errors) {
    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && isset($_POST['reservation_submit'])
    ) {
        if (isset($api_errors) && is_array($api_errors)) {
            $errors = new WP_Error();

            foreach ($api_errors as $api_error) {
                if (isset($api_error['values'])) {
                    foreach ($api_error['values'] as $value) {
                        $errors->add('ohbe_' . $value, $api_error['text']);
                    }
                }
            }

            return $errors;
        }
        if (isset($api_errors) && $api_errors == 'wrongDate') {
            $errors = new WP_Error();
            $errors->add(
                'ohbe_card_expires',
                __('Card expiration date invalid', 'ohbe')
            );

            return $errors;
        }
    }
}

/**
 * Handle reservation page.
 * Redirect to availability or to reservation complete page.
 *
 * @return mixed
 *      array|object Errors from Oscar reservation call response
 *      int When wrong date is set
 *      void When no errors are set
 */
function ohbeReservationFormHandle() {
    wp_enqueue_script('ohbe_reservation');

    // Modify Booking
    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && isset($_POST['modify_submit'])
    ) {
        // REDIRECT TO AVAILABILITY PAGE
        wp_safe_redirect(esc_url_raw(
            OHBE_Availability::renderFormAction(OHBE_AVAILABILITY_PAGE)
        ));
        exit;
    }
    // Booking
    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && isset($_POST['reservation_submit'])
    ) {
        $search_filters = OHBE_Tools::getSearchFilters('Y-m-d');
        $reservation_data = OHBE_Reservation::getReservationFieldValues();
        if ($reservation_data == "wrongDate") {
            return "wrongDate";
        }
        $accos = OHBE_Reservation::getAccommodationStructure();
        $redforts_data = OscarAPI::setReservation(
            $search_filters['check_in'],
            $search_filters['check_out'],
            'confirmed',
            $accos,
            $reservation_data,
            $search_filters['promotion_code']
        );
        if (isset($redforts_data['errors'])) {
            return $redforts_data['errors'];
        }
        $arrival_date = DateTime::createFromFormat(
            '!Y-m-d',
            $search_filters['check_in']
        );
        OHBE_Reservation::setReservationParam('just_reserved', true);
        OHBE_Reservation::setReservationParam('arrival', $arrival_date);
        OHBE_Reservation::setReservationParam('reserv_id', $redforts_data['id']);
        OHBE_Reservation::setReservationParam(
            'email',
            $reservation_data['ohbe_email']
        );
        OHBE_Tools::setFlashMessage(__(
            'An email message has been sent to you. If you do not find '
            . 'it in your Inbox, check the Spam folder and mark the email '
            . '"Not spam".',
            'ohbe'
        ));
        // Redirect to reservation complete
        wp_safe_redirect(esc_url_raw(
            OHBE_Tools::getBaseUrl() . '?' . OHBE_RESERVATION_COMPLETE_PAGE
        ));
        exit;
    }
    if (isset($_GET['ohbe'])) {
        $all_filters = OHBE_Tools::decodeFilters($_GET['ohbe']);
        $errors = OHBE_Availability::accommodationAvailabilityErrors(
            'position',
            $all_filters
        );
        if ($errors) {
            return $errors;
        }
    }
    else {
        $errors = OHBE_Availability::accommodationAvailabilityErrors('params');
        if ($errors) {
            return $errors;
        }
    }
}

add_action('init', 'ohbeReservationFormHandle');

/**
 * Handle reservation complete page forms.
 *
 * @return void
 */
function ohbeReservationCompleteFormHandle() {
    wp_enqueue_script('jquery-ui-accordion');
    wp_enqueue_script('ohbe_reservation_complete');

    if (
        $_SERVER['REQUEST_METHOD'] == "POST"
        && isset($_POST['item_submit'])
        && OHBE_Reservation::getReservationParam()
    ) {
        $redforts_data = OscarAPI::setItemsUpdate(array(
            'id' => intval($_POST['ohbe_id']),
            'quantity' => intval($_POST['ohbe_quantity']),
        ));
        if (isset($redforts_data['errors'])) {
            OHBE_Tools::setFlashMessage(
                $redforts_data['errors'][0]['text'],
                'danger'
            );
        }
        else {
            OHBE_Tools::setFlashMessage(__(
                'An email message has been sent to you. If you do not find '
                . 'it in your Inbox, check the Spam folder and mark the email '
                . '"Not spam".'
                , 'ohbe'
            ));
        }
    }

    // Ensure that reservations params are defined
    if (OHBE_Reservation::getReservationParam()) {
        OHBE_Reservation::setReservationParam('item_list', OscarAPI::getItemList());
        $item_list = OHBE_Reservation::getReservationParam('item_list');
        if (isset($item_list['errors'])) {
            OHBE_Tools::setFlashMessage(
                $item_list['errors'][0]['text'],
                'danger'
            );
            unset($_SESSION['ohbe']['reservation']);
        }
    }
}

add_action('init', 'ohbeReservationCompleteFormHandle');


